/****************************************************************************/
/* CC1000 interface library                                                 */
/*                                                                          */
/* File:      cc1000.c                                                      */
/* Revision:  2.0                                                           */
/*                                                                          */
/* Microcontroller:                                                         */
/*          Microchip PIC16F876                                             */
/* Written for the IAR PIC16 compiler                                       */
/*                                                                          */
/* Author:  Karl H. Torvmark, Field Applications Engineer, Chipcon          */
/*                                                                          */
/* Contact: Chipcon AS +47 22 95 85 44                                      */
/*          wireless@chipcon.com                                            */
/****************************************************************************/

/****************************************************************************/
/* This library contains functions for configuring the CC1000. These        */
/* routines use bit-banging to program the CC1000, faster configuration is  */
/* possible by using a synchronous serial port such as a SPI interface.     */
/* The header file "modemhw.h" contains definitions for the various I/O     */
/* pins, the user should make a similar file to name the pins used to       */
/* communicate with the CC1000. Routines to read and write the calibration  */
/* values in the CC1000 are provided, they are not used in this reference   */
/* application, but are useful in other applications, most notably          */
/* frequency-agile and frequency hopping applications. See application      */
/* note AN009 for more information.                                         */
/* The routines in this file will have to be adapted depending on the MCU   */
/* and compiler used. The method used for shifting data in and out may have */
/* to be changed if the bit ordering for bitfields is different from the    */
/* IAR PIC compiler.                                                        */
/*                                                                          */
/* Configuration routines are included in two versions: one using general   */
/* I/O ports ("bit-banging"), and one using the built-in SPI interface of   */
/* the PIC16F876. If possible, the SPI version should be used, as this is   */
/* much faster. The SPI versions are used if the symbol "SPI" is defined,   */
/* otherwise the general I/O-based version is used.                         */
/****************************************************************************/


#include "io16f876.h"
#include "CC1000.h"
#include "modemhw.h"


// Value of time-out timer during calibration
#define TIMEOUTVALUE 10000

/****************************************************************************/
/*  This routine sends new configuration data to the CC1000                 */
/****************************************************************************/

void ConfigureCC1000(char Count, short Configuration[])
{
  short val;
  char i;
  
  for (i=0;i<Count;i++) {
    val=Configuration[i];
    WriteToCC1000RegisterWord(val);
  }
   
}

/****************************************************************************/
/* SPI versions of configuration routines. The SPI interface must be        */
/* initialised correctly before use                                         */
/****************************************************************************/

#ifdef SPI


/****************************************************************************/
/*  This routine writes to a single CC1000 register                         */
/****************************************************************************/

void WriteToCC1000Register(char addr, char data)
{
  char dummy;

  PALE=0;
   
  dummy=SSPBUF;
  SSPBUF=(addr<<1)|0x01; // Write address to CC1000, write bit is always 1
 
  // Wait until data is written 
  while (BF==0)
    ;    
  PALE=1;
  
  dummy=SSPBUF;  
  SSPBUF=data;
  while (BF==0)
    ;
}

/****************************************************************************/
/*  This routine writes to a single CC1000 register, with data and address  */
/*  given in the same variable                                              */
/****************************************************************************/

void WriteToCC1000RegisterWord(short addranddata)
{
  char dummy;
  
  union {
    unsigned short data;
    struct {
      char LowByte;
      char HighByte;
    };
  };
      
  data=addranddata;
  PALE=0;
   
  dummy=SSPBUF;
  SSPBUF=LowByte|0x01; // Write address to CC1000, write bit is always 1
 
  // Wait until data is written 
  while (BF==0);    
  PALE=1;
  
  dummy=SSPBUF;  
  SSPBUF=HighByte;
  while (BF==0)
    ;
}

/****************************************************************************/
/*  This routine reads from a single CC1000 register                        */
/****************************************************************************/

char ReadFromCC1000Register(char addr)
{

  char Value;
  
 
  PALE=0;
  Value=SSPBUF;
  SSPBUF=(addr<<1)&0xFE; // Write address to CC1000, write bit is always 0
  
  // Wait until data is written 
  while (BF==0);
  SSPOV=0;
  
  // Switch direction 
  PDATA_OUT=1;
  TRISC|=0x20; // Set up PDATAOUT as an input
  
  PALE=1;
  
  SSPBUF=0xFF; // Dummy write 
  
  while (BF==0);
  Value=SSPBUF;
  
  TRISC&=~0x20; // Set PDATAOUT as an output 
  return Value;
}

#else  

/****************************************************************************/
/* General I/O pin "bit-bashing" versions of configuration routines.        */
/****************************************************************************/

/****************************************************************************/
/*  This routine writes to a single CC1000 register                         */
/****************************************************************************/

void WriteToCC1000Register(char addr, char data)
{
  short val;
  
  val=(short) (addr&0x7F)<<9 | (short) data &0x00FF;
  WriteToCC1000RegisterWord(val);
}

/****************************************************************************/
/*  This routine writes to a single CC1000 register, with address and data  */
/*  given in the same variable                                              */
/****************************************************************************/

void WriteToCC1000RegisterWord(short addranddata)
{
  char  BitCounter;
  union {     // This union is used to easily access the most significant
              // bit of the configuration data
              
              // Note : This assumes that the C compiler stores bit-fields
              // with the first field going into the LSB. If this is not the
              // case, move the MSB definition to the first bit
    unsigned short Data;
    struct
    {
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short :1;
      unsigned short MSB :1;
     };
  };
  
  PALE=1;

  Data=addranddata;
  PALE=0;
    
  // Send address bits 
  for (BitCounter=0;BitCounter<7;BitCounter++)
  {
    PCLK=1;
    PDATA_OUT=MSB;
    Data=Data<<1;
    PCLK=0;
  }
  // Send read/write bit 
  // Ignore bit in data, always use 1 
  
  PCLK=1;
  PDATA_OUT=1;
  PCLK=0;
  Data=Data<<1;
  PCLK=1;
  PALE=1;
    
  // Send data bits 
  
  for (BitCounter=0;BitCounter<8;BitCounter++)
  {
    PCLK=1;
    PDATA_OUT=MSB;
    Data=Data<<1;
    PCLK=0;
  }
  PCLK=1;
}

/****************************************************************************/
/*  This routine reads from a single CC1000 register                        */
/****************************************************************************/

char ReadFromCC1000Register(char addr)
{
  char BitCounter;
 
  union {     // This unit is used to easily access the most significant 
              // bit of the configuration data 

              // Note : This assumes that the C compiler stores bit-fields 
              // with the first field going into the LSB. If this is not the 
              // case, switch the MSB and LSB definitions 
    unsigned char Data;
    struct
    {
      unsigned char LSB :1;
      unsigned char :1;
      unsigned char :1;
      unsigned char :1;
      unsigned char :1;
      unsigned char :1;
      unsigned char :1;
      unsigned char MSB :1;
     };
  };
  
  PALE=1;
  
  Data=addr<<1;
  PALE=0;
    
  // Send address bits
  for (BitCounter=0;BitCounter<7;BitCounter++)
  {
    PCLK=1;
    PDATA_OUT=MSB;
    Data=Data<<1;
    PCLK=0;
  }
  // Send read/write bit 
  // Ignore bit in data, always use 0 
  
  PCLK=1;
  PDATA_OUT=0;
  PCLK=0;
 
 
  PCLK=1;
  PALE=1;
    
  // Receive data bits 
  
  PDATA_OUT=1;
   
  TRISC|=0x20; // Set up PDATA as an input
    
  for (BitCounter=0;BitCounter<8;BitCounter++)
  {
    PCLK=0;
    Data=Data<<1;
    LSB=PDATA_IN;
    PCLK=1;
  }
  
  
  TRISC&=~0x20; // Set up PDATA as an output again
  
  return Data;
}

#endif
  
/****************************************************************************/
/*  This routine resets the CC1000, clearing all registers.                 */
/****************************************************************************/  

void ResetCC1000(void)
{
  char MainValue;
  
  MainValue=ReadFromCC1000Register(CC1000_MAIN);
  WriteToCC1000Register(CC1000_MAIN,MainValue & 0xFE);         /* Reset CC1000 */
  WriteToCC1000Register(CC1000_MAIN,MainValue | 0x01);         /* Bring CC1000 out of reset */
}


/****************************************************************************/
/*  This routine calibrates the CC1000                                      */
/*  Returns 0 if calibration fails, non-zero otherwise. Checks the LOCK     */
/*  to check for success.                                                   */
/****************************************************************************/

char CalibrateCC1000(void)
{
  char PAVal;
  
  int TimeOutCounter;



  PAVal=ReadFromCC1000Register(CC1000_PA_POW);  /* Store PA settings */
  WriteToCC1000Register(CC1000_PA_POW,0x00); /* Turn off PA to avoid spurs */
                                             /* during calibration in TX mode */
  WriteToCC1000Register(CC1000_CAL,0xA6); /* Start calibration */
  
  TimeOutCounter=0;
  
  while(((ReadFromCC1000Register(CC1000_CAL)&0x08)==0)&&(TimeOutCounter<TIMEOUTVALUE)) {
    TimeOutCounter++;  // Wait until calibration finished, go through loop 10000 times 
  }
  
  WriteToCC1000Register(CC1000_CAL,0x26); /* End calibration */
  WriteToCC1000Register(CC1000_PA_POW,PAVal); /* Restore PA setting */
  
  return ((ReadFromCC1000Register(CC1000_LOCK)&&0x01==1));
}

/****************************************************************************/
/*  This routine puts the CC1000 into RX mode (from TX). When switching to  */
/*  RX from PD, use WakeupC1000ToRX first                                   */
/****************************************************************************/


void SetupCC1000RX(char RXCurrent)
{
 // WriteToCC1000Register(CC1000_MAIN,0x31);    /* Switch into RX, switch to freq. reg A */
  WriteToCC1000Register(CC1000_CURRENT,RXCurrent); /* Program VCO current for RX */
  WriteToCC1000Register(CC1000_MAIN,0x11);    /* Switch RX part of CC1000 on */
}

/****************************************************************************/
/*  This routine puts the CC1000 into TX mode (from RX). When switching to  */
/*  TX from PD, use WakeupCC1000ToTX first                                  */
/****************************************************************************/

void SetupCC1000TX(char TXCurrent)
{
 // WriteToCC1000Register(CC1000_MAIN,0xF1);    /* Switch into TX, switch to freq. reg B */
  WriteToCC1000Register(CC1000_CURRENT,TXCurrent); /* Program VCO current for TX */
  WriteToCC1000Register(CC1000_MAIN,0xE1);    /* Switch TX part of CC1000 on */
}

/****************************************************************************/
/*  This routine puts the CC1000 into power down mode. Use WakeUpCC1000ToRX */
/*  followed by SetupCC1000RX or WakeupCC1000ToTX followed by SetupCC1000TX */
/*  to wake up from power down                                              */
/****************************************************************************/

void SetupCC1000PD(void)
{
  WriteToCC1000Register(CC1000_MAIN,0x3F);  /* Put CC1000 into power-down */
}

/****************************************************************************/
/*  This routine wakes the CC1000 up from PD mode to RX mode, call          */
/*  SetupCC1000RX after this routine is finished.                           */
/****************************************************************************/

void WakeUpCC1000ToRX(char RXCurrent)
{
  int i;
  
  
  WriteToCC1000Register(CC1000_MAIN,0x3B);  // Turn on xtal oscillator core
  WriteToCC1000Register(CC1000_CURRENT,RXCurrent); // Program VCO current for RX 
  
  // Insert wait routine here, must wait for xtal oscillator to stabilise, 
  // typically takes 2-5ms.

  for (i=0;i<0xFFFF;i++)
    ;
  
  WriteToCC1000Register(CC1000_MAIN,0x39);  // Turn on bias generator
  WriteToCC1000Register(CC1000_MAIN,0x31);  // Turn on frequency synthesiser
}

/****************************************************************************/
/*  This routine wakes the CC1000 up from PD mode to TX mode, call          */
/*  SetupCC1000TX after this routine is finished.                           */
/****************************************************************************/

void WakeUpCC1000ToTX(char TXCurrent)
{
  int i;
  
  
  WriteToCC1000Register(CC1000_MAIN,0xFB);  // Turn on xtal oscillator core
  
  WriteToCC1000Register(CC1000_CURRENT,TXCurrent); // Program VCO current for TX
  
  // Insert wait routine here, must wait for xtal oscillator to stabilise, 
  // typically takes 2-5ms. 
  
  for (i=0;i<0xFFFF;i++)
    ;
  
  WriteToCC1000Register(CC1000_MAIN,0xF9);  // Turn on bias generator
  WriteToCC1000Register(CC1000_MAIN,0xF1);  // Turn on frequency synthesiser
}

/****************************************************************************/
/*  This routine locks the averaging filter of the CC1000                   */
/****************************************************************************/

void AverageManualLockCC1000(void)
{
  WriteToCC1000Register(CC1000_MODEM1,0x19);
}

/****************************************************************************/
/*  This routine unlocks the averaging filter of the CC1000                 */
/****************************************************************************/

void AverageFreeRunCC1000(void)
{
  WriteToCC1000Register(CC1000_MODEM1,0x09);
}

/****************************************************************************/
/*  This routine sets up the averaging filter of the CC1000 for automatic   */
/*  lock. This can be used in polled receivers.                             */
/****************************************************************************/

void AverageAutoLockCC1000(void)
{
  WriteToCC1000Register(CC1000_MODEM1,0x01);
}

/****************************************************************************/
/*  This routine reads the current calibration values from the CC1000       */
/****************************************************************************/

void ReadCurrentCalibration(char *val1, char *val2)
{
  *val1=ReadFromCC1000Register(CC1000_TEST0);
  *val2=ReadFromCC1000Register(CC1000_TEST2);
}

/****************************************************************************/
/*  This routine overrides the current calibration of the CC1000            */
/****************************************************************************/

void OverrideCurrentCalibration(char val1, char val2)
{
  WriteToCC1000Register(CC1000_TEST5,(val1&0x0F)|0x10);
  WriteToCC1000Register(CC1000_TEST6,(val2&0x1F)|0x20);
}

/****************************************************************************/
/*  This routine stops override of the CC1000 calibration values            */
/****************************************************************************/

void StopOverridingCalibration(void)
{
  WriteToCC1000Register(CC1000_TEST5,0x00);
  WriteToCC1000Register(CC1000_TEST6,0x00);
}

